Package: ASSET
Title: An R package for subset-based association analysis of
        heterogeneous traits and subtypes
Version: 2.28.0
Date: 2021-09-04
Authors@R: c(
	person("Samsiddhi", "Bhattacharjee", role = c("aut", "cre"), email="sb1@nibmg.ac.in"),
	person("Guanghao", "Qi", role = "aut"),
	person("Nilanjan", "Chatterjee", role = "aut"),
	person("William", "Wheeler", role = "aut")
	)
Description: An R package for subset-based analysis of heterogeneous traits and disease subtypes. 
	The package allows the user to search through all possible subsets of z-scores to 
	identify the subset of traits giving the best meta-analyzed z-score. Further, it returns a p-value
	adjusting for the multiple-testing involved in the search. It also allows for searching for the 
	best combination of disease subtypes associated with each variant.
Depends: R (>= 3.5.0), stats, graphics
Imports: MASS, msm, rmeta
Suggests: RUnit, BiocGenerics, knitr
VignetteBuilder: knitr
License: GPL-2 + file LICENSE
biocViews: StatisticalMethod, SNP, GenomeWideAssociation,
        MultipleComparison
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/ASSET
git_branch: RELEASE_3_22
git_last_commit: 7481559
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:33:15 UTC; biocbuild
Author: Samsiddhi Bhattacharjee [aut, cre],
  Guanghao Qi [aut],
  Nilanjan Chatterjee [aut],
  William Wheeler [aut]
Maintainer: Samsiddhi Bhattacharjee <sb1@nibmg.ac.in>
